# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: install.py 913 2009-01-02 12:47:38Z mac $
"""Initial generation."""


def evolve(context):
    # nothing to install initially
    pass



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: keyword.py 1001 2009-02-07 08:10:56Z mac $

import persistent
import zope.catalog.interfaces
import zope.container.btree
import zope.container.contained
import zope.component
import zope.interface
import zope.lifecycleevent
import zope.schema.fieldproperty

import icemac.addressbook.interfaces

from icemac.addressbook.i18n import MessageFactory as _


class KeywordContainer(zope.container.btree.BTreeContainer):
    "A container for keywords."
    zope.interface.implements(icemac.addressbook.interfaces.IKeywords)

    def get_keywords(self):
        return sorted(self.values(), key=lambda x: x.title)

    def get_titles(self):
        return [x.title for x in self.get_keywords()]


class Keyword(persistent.Persistent, zope.container.contained.Contained):
    "A keyword."
    zope.interface.implements(icemac.addressbook.interfaces.IKeyword)

    title = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IKeyword['title'])
    notes = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IKeyword['notes'])


@zope.component.adapter(
    icemac.addressbook.interfaces.IKeyword,
    zope.lifecycleevent.IObjectModifiedEvent)
def changed(obj, event):
    for desc in event.descriptions:
        if (desc.interface == icemac.addressbook.interfaces.IKeyword and
            'title' in desc.attributes):
            catalog = zope.component.getUtility(
                zope.catalog.interfaces.ICatalog)
            catalog.updateIndex(catalog.get('keywords'))
            break

@zope.component.adapter(icemac.addressbook.interfaces.IKeyword)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def title(keyword):
    return keyword.title


def uniqueTitles(obj, event):
    if getattr(obj, '__parent__', None) is None:
        return
    container = obj.__parent__
    titles = [x.title for x in container.values()]
    if len(titles) != len(set(titles)):
        raise zope.interface.Invalid(_(u'This keyword already exists.'))



# -*- coding: utf-8 -*-
# Copyright (c) 2009 Michael Howitz
# See also LICENSE.txt
# $Id: sources.py 969 2009-01-25 14:42:26Z mac $

import gocept.reference.interfaces
import zc.sourcefactory.basic
import zc.sourcefactory.contextual
import zope.component
import zope.securitypolicy.interfaces
import zope.app.authentication.principalfolder
import zope.security.proxy

import icemac.addressbook.interfaces

class RoleSource(zc.sourcefactory.basic.BasicSourceFactory):
    """A source for all global roles."""

    def getValues(self):
        roles = zope.component.getAllUtilitiesRegisteredFor(
            zope.securitypolicy.interfaces.IRole)
        names = [r.id
                 for r in roles
                 if r.id.startswith('icemac.addressbook.global.')]
        return sorted(names)

    def getTitle(self, value):
        role = zope.component.getUtility(
            zope.securitypolicy.interfaces.IRole, name=value)
        return role.title

role_source = RoleSource()


class Persons(zc.sourcefactory.contextual.BasicContextualSourceFactory):
    """Persons in addressbook."""

    def getValues(self, context):
        root = icemac.addressbook.principals.interfaces.IRoot(context)
        insecured_context = zope.security.proxy.removeSecurityProxy(context)
        # When editing a user we need this users person in list, as
        # the person on user is not changeable we need only to return
        # this person.
        if isinstance(insecured_context,
                      icemac.addressbook.principals.principals.Principal):
            yield context.person
            return

        for person in root.values():
            if getattr(person.default_email_address, 'email', None):
                # show only persons which are not yet users
                reftarget = gocept.reference.interfaces.IReferenceTarget(person)
                if reftarget.is_referenced(recursive=False):
                    # XXX must check for other types of reference here
                    continue
                yield person

    def getTitle(self, context, value):
        return icemac.addressbook.interfaces.ITitle(value)

persons = Persons()



import os
import sys
import code
import zdaemon.zdctl
import zope.app.wsgi
import zope.app.debug


def application_factory(global_conf, conf='zope.conf'):
    zope_conf = os.path.join(global_conf['here'], conf)
    return zope.app.wsgi.getWSGIApplication(zope_conf)

def interactive_debug_prompt(zope_conf='zope.conf'):
    db = zope.app.wsgi.config(zope_conf)
    debugger = zope.app.debug.Debugger.fromDatabase(db)
    # Invoke an interactive interpreter shell
    banner = ("Welcome to the interactive debug prompt.\n"
              "The 'root' variable contains the ZODB root folder.\n"
              "The 'app' variable contains the Debugger, 'app.publish(path)' "
              "simulates a request.")
    code.interact(banner=banner, local={'debugger': debugger,
                                        'app':      debugger,
                                        'root':     debugger.root()})

class ControllerCommands(zdaemon.zdctl.ZDCmd):

    def do_debug(self, rest):
        interactive_debug_prompt()

    def help_debug(self):
        print "debug -- Initialize the application, providing a debugger"
        print "         object at an interactive Python prompt."

def zdaemon_controller(zdaemon_conf='zdaemon.conf'):
    args = ['-C', zdaemon_conf] + sys.argv[1:]
    zdaemon.zdctl.main(args, options=None, cmdclass=ControllerCommands)

def zdaemon_controller_debug_ajax():
    zdaemon_controller('zdaemon-debug-ajax.conf')
def zdaemon_controller_debug_pdb():
    zdaemon_controller('zdaemon-debug-pdb.conf')



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: token.py 913 2009-01-02 12:47:38Z mac $

import zc.sourcefactory.interfaces
import zope.component
import zope.interface

import icemac.addressbook.export.interfaces

@zope.component.adapter(icemac.addressbook.export.interfaces.IExporter)
@zope.interface.implementer(zc.sourcefactory.interfaces.IToken)
def fromExporter(value):
    "Exporters are global utilities, the existing token adapters do not work."
    return "%s.%s" % (value.__module__, value.__class__.__name__)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: evolve1.py 913 2009-01-02 12:47:38Z mac $

__docformat__ = "reStructuredText"

import zope.app.generations.utility
import zope.app.zopeappgenerations

import icemac.addressbook.interfaces
import icemac.addressbook.addressbook
import icemac.addressbook.person


generation = 1

person_created_with_set_site = icemac.addressbook.utils.set_site(
    icemac.addressbook.person.person_created)

def evolve(context):
    """Installs the necessary components for address books.
    
    Defines references on persons.
    """

    root = zope.app.zopeappgenerations.getRootFolder(context)
    # persons need gocept.reference which needs a local utility stored in root

    # persons
    persons = zope.app.generations.utility.findObjectsProviding(
        root, icemac.addressbook.interfaces.IPerson)
    for person in persons:
        person_created_with_set_site(root, person, None)

    # addressbooks
    addressbooks = zope.app.generations.utility.findObjectsProviding(
        root, icemac.addressbook.interfaces.IAddressBook)
    for addressbook in addressbooks:
        icemac.addressbook.addressbook.create_address_book_infrastructure(
            addressbook, None)




# -*- coding: utf-8 -*-
# Copyright (c) 2009 Michael Howitz
# See also LICENSE.txt

import zope.viewlet.interfaces
import zope.viewlet.manager


class IMasterData(zope.viewlet.interfaces.IViewletManager):
    """Containing viewlets which provide links to edit master data."""


MasterDataManager = zope.viewlet.manager.ViewletManager(
    'master-data', IMasterData, bases=(
        zope.viewlet.manager.WeightOrderedViewletManager,))



# -*- coding: utf-8 -*-
# Copyright (c) 2009 Michael Howitz
# See also LICENSE.txt
# $Id: principals.py 1079 2009-03-23 20:52:14Z mac $

from icemac.addressbook.i18n import MessageFactory as _
import classproperty
import gocept.reference
import icemac.addressbook.interfaces
import icemac.addressbook.principals.interfaces
import zope.annotation.interfaces
import zope.app.authentication.principalfolder
import zope.container.interfaces
import zope.interface
import zope.lifecycleevent
import zope.securitypolicy.interfaces


PASSWORD_MANAGER_NAME = 'SSHA'
EDITOR_VISITOR_PERMS = ('icemac.addressbook.EditPrincipalPassword',
                        'icemac.addressbook.ViewPrincipal',)
ONLY_EDITOR_PERMS = ('icemac.addressbook.EditPrincipal', )


class Principal(zope.app.authentication.principalfolder.InternalPrincipal):
    """Principal where the password manager cannot be specified."""

    zope.interface.implements(
        icemac.addressbook.principals.interfaces.IPrincipal,
        icemac.addressbook.principals.interfaces.IPasswordFields,
        icemac.addressbook.principals.interfaces.IRoles,
        zope.annotation.interfaces.IAttributeAnnotatable)

    _person = gocept.reference.Reference(ensure_integrity=True)
    _login = None
    _password = None
    description = None
    _roles = ()

    def __init__(self):
        self._passwordManagerName = PASSWORD_MANAGER_NAME

    @property
    def title(self):
        # needed by principal folder
        return icemac.addressbook.interfaces.ITitle(self.person)

    class person(classproperty.classproperty):
        def __get__(self):
            return self._person
        def __set__(self, person):
            if self._person is not None:
                # it's not possible to change the person
                return
            self._person = person
            self.login = person.default_email_address.email

    class password(classproperty.classproperty):
        def __get__(self):
            return self._password
        def __set__(self, password):
            if password is not None:
                zope.app.authentication.principalfolder.InternalPrincipal.\
                    setPassword(self, password)

    class password_repetition(classproperty.classproperty):
        def __get__(self):
            pass # password_repetition is not stored
        def __set__(self, password):
            pass # password_repetition is not stored

    class roles(classproperty.classproperty):
        def __get__(self):
            return self._roles
        def __set__(self, roles):
            old_roles = self._roles
            self._roles = roles
            addressbook = icemac.addressbook.principals.interfaces.IRoot(self)
            # Update role map on addressbook
            role_manager = (
                zope.securitypolicy.interfaces.IPrincipalRoleManager(
                    addressbook))
            for role in old_roles:
                role_manager.unsetRoleForPrincipal(role, self.__name__)
            for role in roles:
                role_manager.assignRoleToPrincipal(role, self.__name__)

            # Update permission map on user
            permission_manager = (
                zope.securitypolicy.interfaces.IPrincipalPermissionManager(
                    self))

            for permission in EDITOR_VISITOR_PERMS + ONLY_EDITOR_PERMS:
                permission_manager.unsetPermissionForPrincipal(
                    permission, self.__name__)
            if ('icemac.addressbook.global.Editor' in roles or
                'icemac.addressbook.global.Visitor' in roles):
                for permission in EDITOR_VISITOR_PERMS:
                    permission_manager.grantPermissionToPrincipal(
                        permission, self.__name__)
            if 'icemac.addressbook.global.Editor' in roles:
                for permission in ONLY_EDITOR_PERMS:
                    permission_manager.grantPermissionToPrincipal(
                        permission, self.__name__)


@zope.component.adapter(
    icemac.addressbook.principals.interfaces.IPrincipal,
    zope.lifecycleevent.IObjectCreatedEvent)
def created(principal, event):
    """Create initial infrastructure or update existing infrastructure to
    current requirements (using generation)."""
    # set default values for references as z3c.form accesses the
    # attributes before a value is assigned and gets an AttributeError
    # otherwise
    if not hasattr(principal, '_person'):
        principal._person = None


@zope.component.adapter(
    icemac.addressbook.principals.interfaces.IPrincipal,
    zope.container.interfaces.IObjectAddedEvent)
def added(principal, event):
    # roles need principal.__name__ so we set them again when the
    # principal has been added
    principal.roles = principal._roles


@zope.component.adapter(icemac.addressbook.principals.interfaces.IPrincipal)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def title(principal):
    if getattr(principal, 'person', None) is not None:
        return icemac.addressbook.interfaces.ITitle(principal.person)
    # safty belt:
    return _(u'<no person>')



#! /bin/env python

import os.path
import subprocess
import sys

if __name__ == '__main__':
    python = sys.executable

    # prerequisites
    if os.path.exists('buildout.cfg'):
        print "ERROR: buildout.cfg already exists."
        print "       Please rename the existing one and restart install."
        sys.exit(-1)

    # create admin.zcml
    print ' Log-in name for the administrator: ',
    admin_login = sys.stdin.readline().strip()
    print 'Password for the administrator: ',
    admin_passwd = sys.stdin.readline().strip()
    admin_zcml = file('admin.zcml', 'w')
    admin_zcml.write('\n'.join(
            ('<configure xmlns="http://namespaces.zope.org/zope">',
             '  <principal',
             '    id="icemac.addressbook.global.Administrator"',
             '    title="global administrator"',
             '    login="%s"' % admin_login,
             '    password_manager="Plain Text"',
             '    password="%s" />' % admin_passwd,
             '  <grant',
             '    role="zope.Manager"',
             '    principal="icemac.addressbook.global.Administrator" />',
             '</configure>',
             )))
    admin_zcml.close()

    # create buildout.cfg
    print 'creating buildout.cfg ...'
    buildout_cfg = file('buildout.cfg', 'w')
    buildout_cfg.write('[buildout]\nextends = profiles/prod.cfg')
    buildout_cfg.close()

    # call boostrap.py
    print 'calling %s bootstrap.py ...' % python
    res = subprocess.call([sys.executable, 'bootstrap.py'])
    if res:
        sys.exit(res)

    # run buildout
    res = subprocess.call(['bin/buildout'])
    if res:
        sys.exit(res)



# -*- coding: utf-8 -*-
# Copyright (c) 2009 Michael Howitz
# See also LICENSE.txt
# $Id: interfaces.py 1061 2009-03-19 21:09:55Z mac $

import copy
import zope.interface
import zope.schema

import icemac.addressbook.principals.sources

from icemac.addressbook.i18n import MessageFactory as _


class IRequiredPasswordFields(zope.interface.Interface):
    """Required password fields to enter and check password."""

    password = zope.schema.Password(
        title=_(u'Password'), min_length=8,
        description=_(u'The password for the user.'))

    password_repetition = zope.schema.Password(
        title=_(u'Password repetition'), min_length=8,
        description=_(u'Please repeat the password.'))

    @zope.interface.invariant
    def password_eq_repetition(obj):
        if obj.password != obj.password_repetition:
            raise zope.interface.Invalid(
                _(u'Entry in password field was not equal to entry in password '
                  u'repetition field.'))


class IPasswordFields(IRequiredPasswordFields):
    """Not-required Password fields to enter and check password."""

    password = copy.copy(IRequiredPasswordFields['password'])
    password_repetition = copy.copy(
        IRequiredPasswordFields['password_repetition'])

IPasswordFields['password'].required = False
IPasswordFields['password_repetition'].required = False


class IPrincipal(zope.interface.Interface):
    "Derived from zope.app.authentication.principalfolder.IInternalPrincipal."

    person = zope.schema.Choice(
        title=_(u'Person'), readonly=True,
        source=icemac.addressbook.principals.sources.persons)

    login = zope.schema.TextLine(
        title=_(u'Login'),
        description=_(u'The Login/Username of the user. '
                      u'This value can change.'))

    description = zope.schema.Text(
        title=_('Notes'),
        description=_('Provides notes for the user.'),
        required=False, missing_value='', default=u'')


class IRoles(zope.interface.Interface):
    "Roles"

    roles = zope.schema.Tuple(
        title=_(u'Roles'), required=False,
        value_type=zope.schema.Choice(
            source=icemac.addressbook.principals.sources.role_source))


class IRoot(zope.interface.Interface):
    """Root object where on which global roles are stored.

    You have to provide an adapter from the root object to this interface.
    """



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: add.py 913 2009-01-02 12:47:38Z mac $

import icemac.addressbook.interfaces
import icemac.addressbook.browser.base
import icemac.addressbook.addressbook
from icemac.addressbook.i18n import MessageFactory as _


class AddForm(icemac.addressbook.browser.base.BaseAddForm):
    
    label = _(u'Add new address book')
    interface = icemac.addressbook.interfaces.IAddressBook
    class_ = icemac.addressbook.addressbook.AddressBook
    next_url = 'object'



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: tests.py 1066 2009-03-20 17:13:24Z mac $

import icemac.addressbook.testing


def test_suite():
    return icemac.addressbook.testing.FunctionalDocFileSuite(
        "browser/addressbook/addressbook.txt",
        "browser/authentication/login.txt",
        "browser/export/export.txt",
        "browser/keyword/keyword.txt",
        "browser/masterdata/masterdata.txt",
        "browser/person/person.txt",
        "browser/principals/principals.txt",
        "browser/rootfolder/rootfolder.txt",
        "browser/search/search.txt",
        )



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: menu.py 1069 2009-03-20 17:23:09Z mac $

import zope.viewlet.manager
import z3c.menu.ready2go
import z3c.menu.ready2go.manager

MainMenu = zope.viewlet.manager.ViewletManager(
    'main', z3c.menu.ready2go.ISiteMenu,
    bases=(z3c.menu.ready2go.manager.MenuManager,))

AddMenu = zope.viewlet.manager.ViewletManager(
    'add', z3c.menu.ready2go.IAddMenu,
    bases=(z3c.menu.ready2go.manager.MenuManager,))



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: address.py 1104 2009-03-31 19:53:53Z mac $

import zope.traversing.api

import icemac.addressbook.address
import icemac.addressbook.browser.base
import icemac.addressbook.interfaces

from icemac.addressbook.i18n import MessageFactory as _


class DefaultsDeleteForm(icemac.addressbook.browser.base.BaseDeleteForm):

    next_url = 'parent'

    def _do_delete(self):
        parent = zope.traversing.api.getParent(self.context)
        prefix = icemac.addressbook.address.object_to_prefix(self.context)
        default_attr = 'default_' + prefix
        default_obj = getattr(parent, default_attr)
        if self.context == default_obj:
            replacement_obj = icemac.addressbook.utils.create_obj(
                icemac.addressbook.address.object_to_class(self.context))
            icemac.addressbook.utils.add(parent, replacement_obj)
            setattr(parent, default_attr, replacement_obj)
        super(DefaultsDeleteForm, self)._do_delete()


class AddPhoneNumberForm(icemac.addressbook.browser.base.BaseAddForm):

    interface = icemac.addressbook.interfaces.IPhoneNumber
    class_ = icemac.addressbook.address.PhoneNumber
    next_url = 'parent'


class DeletePhoneNumberForm(DefaultsDeleteForm):

    label = _(u'Do you really want to delete this phone number?')
    interface = icemac.addressbook.interfaces.IPhoneNumber


class AddPostalAddressForm(icemac.addressbook.browser.base.BaseAddForm):

    interface = icemac.addressbook.interfaces.IPostalAddress
    class_ = icemac.addressbook.address.PostalAddress
    next_url = 'parent'


class DeletePostalAddressForm(DefaultsDeleteForm):

    label = _(u'Do you really want to delete this postal address?')
    interface = icemac.addressbook.interfaces.IPostalAddress


class AddEMailAddressForm(icemac.addressbook.browser.base.BaseAddForm):

    interface = icemac.addressbook.interfaces.IEMailAddress
    class_ = icemac.addressbook.address.EMailAddress
    next_url = 'parent'


class DeleteEMailAddressForm(DefaultsDeleteForm):

    label = _(u'Do you really want to delete this e-mail address?')
    interface = icemac.addressbook.interfaces.IEMailAddress


class AddHomePageAddressForm(icemac.addressbook.browser.base.BaseAddForm):

    interface = icemac.addressbook.interfaces.IHomePageAddress
    class_ = icemac.addressbook.address.HomePageAddress
    next_url = 'parent'


class DeleteHomePageAddressForm(DefaultsDeleteForm):

    label = _(u'Do you really want to delete this home page address?')
    interface = icemac.addressbook.interfaces.IHomePageAddress



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: keyword.py 1037 2009-03-05 17:52:10Z mac $

from icemac.addressbook.i18n import MessageFactory as _
import gocept.reference.interfaces
import icemac.addressbook.browser.base
import icemac.addressbook.interfaces
import z3c.form.button
import zope.component
import zope.traversing.browser.interfaces


class AddForm(icemac.addressbook.browser.base.BaseAddForm):
    
    label = _(u'Add new keyword')
    interface = icemac.addressbook.interfaces.IKeyword
    class_ = icemac.addressbook.keyword.Keyword
    next_url = 'parent'


def can_delete_keyword(form):
    """Button condition telling if the displayed keyword is deleteable."""
    return (
        icemac.addressbook.browser.base.can_access('@@delete.html')(form)
        and
        not gocept.reference.interfaces.IReferenceTarget(
            form.context).is_referenced()
        )

class EditForm(icemac.addressbook.browser.base.BaseEditForm):

    label = _(u'Edit keyword')
    interface = icemac.addressbook.interfaces.IKeyword
    next_url = 'parent'

    @z3c.form.button.buttonAndHandler(_('Apply'), name='apply')
    def handleApply(self, action):
        # because we define a new action we have to duplicate the
        # existing action because otherwise we'll loose it.
        super(EditForm, self).handleApply(self, action)

    @z3c.form.button.buttonAndHandler(_('Cancel'), name='cancel')
    def handleCancel(self, action):
        self.status = self.noChangesMessage

    @z3c.form.button.buttonAndHandler(
        _(u'Delete'), name='delete', condition=can_delete_keyword)
    def handleDelete(self, action):
        url = zope.component.getMultiAdapter(
            (self.context, self.request),
            zope.traversing.browser.interfaces.IAbsoluteURL)()
        self.request.response.redirect(url + '/@@delete.html')


class DeleteForm(icemac.addressbook.browser.base.BaseDeleteForm):
    label = _(u'Do you really want to delete this keyword?')
    interface = icemac.addressbook.interfaces.IKeyword
    field_names = ('title', )



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: person.py 1081 2009-03-28 20:03:50Z mac $

from icemac.addressbook.address import interface_title_prefix_class_mapping
from icemac.addressbook.i18n import MessageFactory as _
import gocept.reference.interfaces
import icemac.addressbook.browser.base
import icemac.addressbook.interfaces
import icemac.addressbook.person
import icemac.addressbook.sources
import icemac.addressbook.utils
import z3c.form.button
import z3c.form.datamanager
import z3c.form.group
import zc.sourcefactory.contextual
import zope.component
import zope.traversing.browser.interfaces

def prefix_to_class(prefix):
    mapping = dict((x[2], x[3]) for x in interface_title_prefix_class_mapping)
    return mapping[prefix]


class AddGroup(icemac.addressbook.browser.base.PrefixGroup):
    "PrefixGroup for AddForm."

    def __init__(self, context, request, parent, interface, label, prefix):
        super(AddGroup, self).__init__(context, request, parent)
        self.interface = interface
        self.label = label
        self.prefix = prefix


class EditGroup(AddGroup):
    "PrefixGroup for EditForm."

    def __init__(
        self, context, request, parent, interface, label, prefix, index, key):
        super(EditGroup, self).__init__(
            context, request, parent, interface, label, prefix)
        self.prefix = "%s_%s" % (prefix, index)
        self.key = key

    def getContent(self):
        return self.context[self.key]

class DefaultSelectGroup(icemac.addressbook.browser.base.PrefixGroup):
    """Group to select the default addresses."""

    interface = icemac.addressbook.interfaces.IPersonDefaults
    label = _(u'main adresses and numbers')
    prefix = 'defaults'


class PersonAddForm(z3c.form.group.GroupForm,
                    icemac.addressbook.browser.base.BaseAddForm):

    label = _(u'Add new person')
    interface = icemac.addressbook.interfaces.IPerson
    next_url = 'parent'

    def __init__(self, *args, **kw):
        super(PersonAddForm, self).__init__(*args, **kw)
        context = self.context
        request = self.request
        groups = [AddGroup(context, request, self, iface, _(title), prefix)
                  for iface, title, prefix, dummy in
                      interface_title_prefix_class_mapping]
        self.groups = tuple(groups)

    def createAndAdd(self, data):
        person = icemac.addressbook.browser.base.create(
            self, icemac.addressbook.person.Person, data)
        self._name = icemac.addressbook.utils.add(self.context, person)
        for group in self.groups:
            obj = icemac.addressbook.browser.base.create(
                group, prefix_to_class(group.prefix), data)
            icemac.addressbook.utils.add(person, obj)
            # handling of default addresses: the first address is
            # saved as default
            default_attrib = 'default_%s' % group.prefix
            if getattr(person, default_attrib, None) is None:
                setattr(person, default_attrib, obj)
        return person


def person_deletable(form):
    "Button display constraint which checks whether a person is deleteable."
    person = form.context
    ref_target = gocept.reference.interfaces.IReferenceTarget(person)
    return not ref_target.is_referenced(recursive=False)


class PersonEditForm(
    z3c.form.group.GroupForm, icemac.addressbook.browser.base.BaseEditForm):

    label = _(u'Edit person data')
    interface = icemac.addressbook.interfaces.IPerson
    next_url = 'parent'

    def __init__(self, *args, **kw):
        super(PersonEditForm, self).__init__(*args, **kw)
        groups = [DefaultSelectGroup]
        for iface, title, prefix, dummy in interface_title_prefix_class_mapping:
            index = 0
            default_obj = getattr(self.context, 'default_%s' % prefix)
            for obj in icemac.addressbook.utils.iter_by_interface(
                    self.context, iface):
                if obj == default_obj:
                    obj_title = u'main ' + title
                else:
                    obj_title = u'other ' + title
                group = EditGroup(
                    self.context, self.request, self, iface, _(obj_title),
                    prefix, index, obj.__name__)
                groups.append(group)
                index += 1
        self.groups = tuple(groups)

    @z3c.form.button.buttonAndHandler(_('Apply'), name='apply')
    def handleApply(self, action):
        # because we define a new action we have to duplicate the
        # existing action because otherwise we'll loose it.
        super(PersonEditForm, self).handleApply(self, action)

    @z3c.form.button.buttonAndHandler(_('Cancel'), name='cancel')
    def handleCancel(self, action):
        self.status = self.noChangesMessage

    @z3c.form.button.buttonAndHandler(
        _(u'Delete person'), name='delete_person',
        condition=icemac.addressbook.browser.base.all_(
            icemac.addressbook.browser.base.can_access('@@delete_person.html'),
            person_deletable))
    def handleDeletePerson(self, action):
        url = zope.component.getMultiAdapter(
            (self.context, self.request),
            zope.traversing.browser.interfaces.IAbsoluteURL)()
        self.request.response.redirect(url + '/@@delete_person.html')

    @z3c.form.button.buttonAndHandler(
        _(u'Delete address or number'), name='delete_address',
        condition=icemac.addressbook.browser.base.can_access(
            '@@delete_address.html'))
    def handleDeleteAddress(self, action):
        url = zope.component.getMultiAdapter(
            (self.context, self.request),
            zope.traversing.browser.interfaces.IAbsoluteURL)()
        self.request.response.redirect(url + '/@@delete_address.html')


class KeywordDataManager(z3c.form.datamanager.AttributeField):
    """Datamanager which converts the list of keywords in the view into a
    set of keywords on the model."""

    def get(self):
        return sorted(super(KeywordDataManager, self).get(),
                      key=lambda x: x.title)

    def set(self, value):
        return super(KeywordDataManager, self).set(set(value))


class DeletePersonForm(icemac.addressbook.browser.base.BaseDeleteForm):
    label = _(u'Do you really want to delete this person?')
    interface = icemac.addressbook.interfaces.IPerson
    field_names = ('first_name', 'last_name')

    def _do_delete(self):
        try:
            super(DeletePersonForm, self)._do_delete()
        except gocept.reference.interfaces.IntegrityError:
            pass # XXX abort transaction (IntegrityError dooms it) and
                 # set notice using z3c.flashmessage


class PersonEntriesSource(
    zc.sourcefactory.contextual.BasicContextualSourceFactory):
    "Source to select entries from person."

    def getValues(self, context):
        for interface, d, d, d in interface_title_prefix_class_mapping:
            for value in icemac.addressbook.sources.ContextByInterfaceSource(
                interface).factory.getValues(context):
                yield value

    def getTitle(self, context, value):
        title_prefix = icemac.addressbook.address.object_to_title(value)
        title = icemac.addressbook.interfaces.ITitle(value)
        if title_prefix:
            title = '%s -- %s' % (title_prefix, title)
        return title


class IPersonEntries(zope.interface.Interface):
    """Content entries of an object."""

    entry = zope.schema.Choice(
        title=_(u'Entries'), source=PersonEntriesSource())


class DeleteAddressForm(icemac.addressbook.browser.base.BaseEditForm):

    label = _(u'Please choose an entry for deletion:')
    interface = IPersonEntries
    ignoreContext = True
    next_url = 'object'

    @z3c.form.button.buttonAndHandler(_(u'Delete entry'), name='delete_entry')
    def handleDeleteEntry(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        selected_entry = data['entry']

        url = zope.component.getMultiAdapter(
            (selected_entry, self.request),
            zope.traversing.browser.interfaces.IAbsoluteURL)()
        self.request.response.redirect(url + '/@@delete.html')

    @z3c.form.button.buttonAndHandler(_(u'Cancel'), name='cancel')
    def handleCancel(self, action):
        self.status = self.noChangesMessage



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: interfaces.py 913 2009-01-02 12:47:38Z mac $

import zope.interface

class IExporter(zope.interface.Interface):
    """Exporting facility."""

    title = zope.interface.Attribute(u'Title of the exporter.')
    description = zope.interface.Attribute(
        u'Short description of the exporter.')
    file_extension = zope.interface.Attribute(
        u'Extension (without the leading dot!) to be set on export file name.')
    mime_type = zope.interface.Attribute(u'Mime-type of the export file.')

    def export(*persons):
        """Export the `persons` to a file.

        Returns a file or file-like-object.

        """



# -*- coding: utf-8 -*-
# Copyright (c) 2009 Michael Howitz
# See also LICENSE.txt
# $Id: principals.py 1079 2009-03-23 20:52:14Z mac $

from icemac.addressbook.i18n import MessageFactory as _
import icemac.addressbook.browser.base
import icemac.addressbook.browser.table
import icemac.addressbook.principals.interfaces
import icemac.addressbook.principals.principals
import transaction
import z3c.form.button
import z3c.form.field
import z3c.pagelet.browser
import z3c.table.table
import z3c.table.value
import zope.app.authentication.principalfolder
import zope.interface
import zope.schema
import zope.schema.interfaces
import zope.security
import zope.traversing.browser.interfaces

class Overview(z3c.pagelet.browser.BrowserPagelet,
               icemac.addressbook.browser.table.Table):

    no_rows_message = _(
        u'No users defined yet or you are not allowed to access any.')

    update = icemac.addressbook.browser.table.Table.update

    def setUpColumns(self):
        return [
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.TitleLinkColumn,
                'user', weight=1),
            z3c.table.column.addColumn(
                self, z3c.table.column.GetAttrColumn, 'login', weight=2,
                header=_(u'Login'), attrName='login'),
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.TruncatedContentColumn,
                'notes', weight=3,
                header=_(u'Notes'), attrName='description', length=50),
            ]

    @property
    def values(self):
        for principal in self.context.values():
            if zope.security.canAccess(principal, 'login'):
                yield principal


class PersonFieldDataManager(z3c.form.datamanager.AttributeField):
    """Person is a readonly field which should be written once."""

    zope.component.adapts(
        icemac.addressbook.principals.interfaces.IPrincipal,
        zope.schema.interfaces.IChoice)

    def set(self, value):
        if self.context.person is not None:
            # was already set --> get error message from parent class
            super(PersonFieldDataManager, self).set(value)
        self.context.person = value


class AddForm(icemac.addressbook.browser.base.BaseAddForm):

    label = _(u'Add new user')
    class_ = icemac.addressbook.principals.principals.Principal
    next_url = 'parent'
    fields = (
        z3c.form.field.Fields(
            icemac.addressbook.principals.interfaces.IPrincipal).select(
            'person')
        +
        z3c.form.field.Fields(
            icemac.addressbook.principals.interfaces.IRequiredPasswordFields)
        +
        z3c.form.field.Fields(
            icemac.addressbook.principals.interfaces.IRoles)
        +
        z3c.form.field.Fields(
            icemac.addressbook.principals.interfaces.IPrincipal).omit(
            'login', 'person')
        )

    def add(self, obj):
        try:
            return super(AddForm, self).add(obj)
        except zope.container.interfaces.DuplicateIDError, e:
            transaction.doom()
            raise z3c.form.interfaces.ActionExecutionError(
                zope.interface.Invalid(_(e.args[0])))


class EditForm(icemac.addressbook.browser.base.BaseEditFormWithCancel):

    next_url = 'parent'
    fields = (
        z3c.form.field.Fields(
            icemac.addressbook.principals.interfaces.IPrincipal).select(
            'person', 'login')
        +
        z3c.form.field.Fields(
            icemac.addressbook.principals.interfaces.IPasswordFields)
        +
        z3c.form.field.Fields(
            icemac.addressbook.principals.interfaces.IRoles)
        +
        z3c.form.field.Fields(
            icemac.addressbook.principals.interfaces.IPrincipal).omit(
            'person', 'login')
        )

    @z3c.form.button.buttonAndHandler(_('Apply'), name='apply')
    def handleApply(self, action):
        # because we define a new action we have to duplicate the
        # existing action because otherwise we'll loose it.
        super(EditForm, self).handleApply(self, action)

    @z3c.form.button.buttonAndHandler(_('Cancel'), name='cancel')
    def handleCancel(self, action):
        super(EditForm, self).handleCancel(self, action)

    @z3c.form.button.buttonAndHandler(
        _(u'Delete user'), name='delete_user',
        condition=icemac.addressbook.browser.base.can_access(
            '@@delete_user.html'))
    def handleDeleteUser(self, action):
        url = zope.component.getMultiAdapter(
            (self.context, self.request),
            zope.traversing.browser.interfaces.IAbsoluteURL)()
        self.request.response.redirect(url + '/@@delete_user.html')

    def applyChanges(self, data):
        try:
            super(EditForm, self).applyChanges(data)
        except ValueError, e:
            transaction.doom()
            raise z3c.form.interfaces.ActionExecutionError(
                zope.interface.Invalid(_(e.args[0])))


class DeleteUserForm(icemac.addressbook.browser.base.BaseDeleteForm):
    label = _(u'Do you really want to delete this user?')
    interface = icemac.addressbook.principals.interfaces.IPrincipal
    field_names = ('person', 'login')



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: simple.py 913 2009-01-02 12:47:38Z mac $

import zope.session.interfaces

import icemac.addressbook.interfaces


class SimpleExport(object):

    def __call__(self):
        session = zope.session.interfaces.ISession(self.request)[
            icemac.addressbook.interfaces.PACKAGE_ID]
        persons = [self.context[id] for id in session['person_ids']]
        exporter = icemac.addressbook.sources.exporter_source.factory.getValue(
            icemac.addressbook.sources.exporter_source, 
            session['exporter_token'])

        self.request.response.setHeader('Content-Type', exporter.mime_type)
        self.request.response.setHeader(
            'Content-Disposition', 
            'attachment; filename=addressbook_export.%s' % (
                exporter.file_extension))

        return exporter.export(*persons)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: address.py 997 2009-02-03 16:58:46Z mac $

import persistent
import zope.container.contained
import zope.interface
import zope.schema.fieldproperty

import icemac.addressbook.interfaces

from icemac.addressbook.i18n import MessageFactory as _


class PostalAddress(
    persistent.Persistent, zope.container.contained.Contained):
    "A postal address."

    zope.interface.implements(icemac.addressbook.interfaces.IPostalAddress)

    kind = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPostalAddress['kind'])
    address_prefix = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPostalAddress['address_prefix'])
    street = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPostalAddress['street'])
    city = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPostalAddress['city'])
    zip = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPostalAddress['zip'])
    state = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPostalAddress['state'])
    country = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPostalAddress['country'])
    notes = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPostalAddress['notes'])


def get_kind_title(obj, iface):
    """Get the title for the kind attribute of an object.

    iface: ... interface, used for look up title of kind's value
    """

    if obj.kind is None:
        title = _(u'unknown')
    else:
        title = zope.component.getMultiAdapter(
            (iface['kind'], obj.kind), icemac.addressbook.interfaces.ITitle)
    return title


@zope.component.adapter(icemac.addressbook.interfaces.IPostalAddress)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def postal_address_title(address):
    """Title of a postal address."""
    title = get_kind_title(
        address, icemac.addressbook.interfaces.IPostalAddress)
    values = [icemac.addressbook.interfaces.ITitle(getattr(address, x))
              for x in ('address_prefix', 'street', 'zip', 'city', 'state',
                        'country')
              if getattr(address, x)]
    if values:
        title += ': ' + ', '.join(values)
    return title

class EMailAddress(
    persistent.Persistent, zope.container.contained.Contained):
    """An e-mail address."""

    zope.interface.implements(icemac.addressbook.interfaces.IEMailAddress)

    kind = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IEMailAddress['kind'])
    email = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IEMailAddress['email'])
    notes = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IEMailAddress['notes'])


@zope.component.adapter(icemac.addressbook.interfaces.IEMailAddress)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def email_address_title(email):
    """Title of an e-mail address."""
    title = get_kind_title(email, icemac.addressbook.interfaces.IEMailAddress)
    if email.email:
        title += ': %s' % email.email
    return title


class HomePageAddress(
    persistent.Persistent, zope.container.contained.Contained):
    """A home page address."""

    zope.interface.implements(icemac.addressbook.interfaces.IHomePageAddress)

    kind = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IHomePageAddress['kind'])
    url = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IHomePageAddress['url'])
    notes = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IHomePageAddress['notes'])


@zope.component.adapter(icemac.addressbook.interfaces.IHomePageAddress)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def home_page_address_title(hp):
    """Title of a home page address."""
    title = get_kind_title(hp, icemac.addressbook.interfaces.IHomePageAddress)
    if hp.url:
        title += ': %s' % hp.url
    return title


class PhoneNumber(
    persistent.Persistent, zope.container.contained.Contained):
    """A phone number."""

    zope.interface.implements(icemac.addressbook.interfaces.IPhoneNumber)

    kind = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPhoneNumber['kind'])
    number = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPhoneNumber['number'])
    notes = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPhoneNumber['notes'])


@zope.component.adapter(icemac.addressbook.interfaces.IPhoneNumber)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def phone_number_title(tel):
    """Title of a phone number."""
    title = get_kind_title(tel, icemac.addressbook.interfaces.IPhoneNumber)
    if tel.number:
        title += ': %s' % tel.number
    return title


interface_title_prefix_class_mapping = (
    (icemac.addressbook.interfaces.IPostalAddress, u'postal address',
     'postal_address', PostalAddress),
    (icemac.addressbook.interfaces.IPhoneNumber, u'phone number',
     'phone_number', PhoneNumber),
    (icemac.addressbook.interfaces.IEMailAddress, u'e-mail address',
     'email_address', EMailAddress),
    (icemac.addressbook.interfaces.IHomePageAddress, u'home page address',
     'home_page_address', HomePageAddress),
    )


def object_to_prefix(obj):
    """Convert an object to its prefix."""
    for interface, d, prefix, d in interface_title_prefix_class_mapping:
        if interface.providedBy(obj):
            return prefix
    raise KeyError(obj)


def object_to_title(obj):
    """Convert an object to its title."""
    for interface, title, d, d in interface_title_prefix_class_mapping:
        if interface.providedBy(obj):
            return title
    raise KeyError(obj)


def object_to_class(obj):
    """Convert an object to its class."""
    for interface, d, d, class_ in interface_title_prefix_class_mapping:
        if interface.providedBy(obj):
            return class_
    raise KeyError(obj)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: utils.py 997 2009-02-03 16:58:46Z mac $

import zope.container.interfaces
import zope.event
import zope.lifecycleevent
import zope.site.hooks


def set_site(func):
    "Decorator which does the set-site-dance."
    def decorated(site, *args, **kw):
        old_site = zope.site.hooks.getSite()
        try:
            zope.site.hooks.setSite(site)
            return func(*args, **kw)
        finally:
            zope.site.hooks.setSite(old_site)
    return decorated


def create_obj(class_, *args, **kw):
    """Create an object of class and fire created event."""
    obj = class_(*args)
    zope.event.notify(zope.lifecycleevent.ObjectCreatedEvent(obj))
    for attrib, value in kw.items():
        setattr(obj, attrib, value)
    return obj


create_obj_with_set_site = set_site(create_obj)


def add(parent, obj):
    nc = zope.container.interfaces.INameChooser(parent)
    name = nc.chooseName('', obj)
    parent[name] = obj
    return name


def create_and_add(parent, class_, *args, **kw):
    obj = create_obj(class_, *args, **kw)
    return add(parent, obj)


create_and_add_with_set_site = set_site(create_and_add)


def iter_by_interface(container, interface):
    "Iterate a container and return only objects providing the specified iface."
    for obj in container.values():
        if interface.providedBy(obj):
            yield obj




# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: evolve2.py 1001 2009-02-07 08:10:56Z mac $

__docformat__ = "reStructuredText"

import zope.app.generations.utility
import zope.app.zopeappgenerations
import zope.catalog.interfaces

import icemac.addressbook.interfaces


generation = 2


@icemac.addressbook.utils.set_site
def fix_kexword_index():
    catalog = zope.component.getUtility(zope.catalog.interfaces.ICatalog)
    catalog.updateIndex(catalog.get('keywords'))


def evolve(context):
    """The steet attribute of the postal address has been split into 2 fields.

    Update keyword index to fix bug that index was not updated, when
    keyword title changed.

    """

    root = zope.app.zopeappgenerations.getRootFolder(context)

    # fix steet attribute
    addresses = zope.app.generations.utility.findObjectsProviding(
        root, icemac.addressbook.interfaces.IPostalAddress)
    for address in addresses:
        if address.street is None:
            continue
        old_street = address.street.split('\n')
        if len(old_street) > 1:
            address.address_prefix = old_street[0].strip()
            address.street = old_street[1].strip()

    # fix title index
    addressbooks = zope.app.generations.utility.findObjectsProviding(
        root, icemac.addressbook.interfaces.IAddressBook)
    for ab in addressbooks:
        fix_kexword_index(ab)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: addressbook.py 1089 2009-03-31 16:03:01Z mac $

import zc.catalog.catalogindex
import zope.app.appsetup.bootstrap
import zope.app.authentication.authentication
import zope.app.authentication.interfaces
import zope.app.authentication.principalfolder
import zope.catalog.catalog
import zope.catalog.interfaces
import zope.authentication.interfaces
import zope.component
import zope.container.btree
import zope.container.interfaces
import zope.interface
import zope.intid
import zope.intid.interfaces
import zope.location
import zope.location.interfaces
import zope.schema.fieldproperty
import zope.site.site
import icemac.addressbook.interfaces
import icemac.addressbook.keyword
import icemac.addressbook.utils


class AddressBook(zope.container.btree.BTreeContainer,
                  zope.site.site.SiteManagerContainer):
    "An address book."

    zope.interface.implements(icemac.addressbook.interfaces.IAddressBook)

    keywords = None
    principals = None

    title = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IAddressBook['title'])
    notes = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPerson['notes'])

@zope.component.adapter(
    icemac.addressbook.interfaces.IAddressBook,
    zope.container.interfaces.IObjectAddedEvent)
def create_address_book_infrastructure(addressbook, event):
    """Create initial infrastructure or update existing infrastructure to
    current requirements (using generation)."""
    # add site manager
    if not zope.location.interfaces.ISite.providedBy(addressbook):
        site_manager = zope.site.site.LocalSiteManager(addressbook)
        addressbook.setSiteManager(site_manager)

    site_manager = addressbook.getSiteManager()

    # add keywords container
    if addressbook.keywords is None:
        keywords = icemac.addressbook.utils.create_obj(
            icemac.addressbook.keyword.KeywordContainer)
        addressbook.keywords = keywords
        zope.location.locate(keywords, addressbook, '++attribute++keywords')
        site_manager.registerUtility(
            keywords, icemac.addressbook.interfaces.IKeywords)

    add_more_addressbook_infrastructure(addressbook, addressbook)


def utility_locally_registered(site, interface):
    """Test, whether a utility is registered on the site manager of the site.

    interface ... interface the utility is registered for

    """

    for registration in site.getSiteManager().registeredUtilities():
        if registration.provided == interface:
            return True
    return False


@icemac.addressbook.utils.set_site
def add_more_addressbook_infrastructure(addressbook):
    "Add more infrastructure which depends on addressbook set as site."
    # intid utility
    if not utility_locally_registered(addressbook,
                                      zope.intid.interfaces.IIntIds):
        intids = zope.app.appsetup.bootstrap.ensureUtility(
            addressbook, zope.intid.interfaces.IIntIds, '', zope.intid.IntIds)

        # register existing persons with intid utility
        for person in addressbook.values():
            intids.register(person)
            # register adresses of persons
            for value in person.values():
                intids.register(value)

    # catalog
    if utility_locally_registered(addressbook,
                                  zope.catalog.interfaces.ICatalog):
        catalog = zope.component.queryUtility(zope.catalog.interfaces.ICatalog)
    else:
        # add catalog
        catalog = zope.app.appsetup.bootstrap.ensureUtility(
            addressbook, zope.catalog.interfaces.ICatalog, '',
            zope.catalog.catalog.Catalog)

    # indexes
    if 'keywords' not in catalog:
        catalog['keywords'] = zc.catalog.catalogindex.SetIndex(
            'get_titles', icemac.addressbook.interfaces.IKeywords,
            field_callable=True)

    # authenticator (PAU)
    if not utility_locally_registered(
        addressbook, zope.authentication.interfaces.IAuthentication):
        pau = zope.app.appsetup.bootstrap.ensureUtility(
            addressbook, zope.authentication.interfaces.IAuthentication, '',
            zope.app.authentication.authentication.PluggableAuthentication)
        pau.credentialsPlugins = (u'No Challenge if Authenticated',
                                  u'Session Credentials',)

    # principals folder
    if addressbook.principals is None:
        principals = icemac.addressbook.utils.create_obj(
            zope.app.authentication.principalfolder.PrincipalFolder)
        addressbook.principals = principals
        zope.location.locate(principals, addressbook, '++attribute++principals')
        site_manager = addressbook.getSiteManager()
        site_manager.registerUtility(
            principals, zope.app.authentication.interfaces.IAuthenticatorPlugin,
            name=u'icemac.addressbook.principals')
    pau.authenticatorPlugins = (u'icemac.addressbook.principals',)



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: table.py 1047 2009-03-14 09:35:01Z mac $

import z3c.form.button
import z3c.table.column
import z3c.table.table
import zc.sourcefactory.interfaces
import zope.interface
import zope.schema
import zope.session.interfaces
import icemac.truncatetext

import icemac.addressbook.browser.base
import icemac.addressbook.interfaces

from icemac.addressbook.i18n import MessageFactory as _


# Columns

class TitleLinkColumn(z3c.table.column.LinkColumn):
    """Column containing the title of an object and a link to the object."""

    header = _(u'Name')
    weight = 2

    def getSortKey(self, item):
        return icemac.addressbook.interfaces.ITitle(item)

    def getLinkContent(self, item):
        return icemac.addressbook.interfaces.ITitle(item)


class TruncatedContentColumn(z3c.table.column.GetAttrColumn):

    length = 20
    attrName = None

    def getValue(self, obj):
        value = super(TruncatedContentColumn, self).getValue(obj)
        return icemac.truncatetext.truncate(value, self.length)


# Tables

class Table(z3c.table.table.Table):
    "Table which supports a no-rows-found message."

    no_rows_message = u'' # Set at subclass.

    def renderTable(self):
        if self.rows:
            return super(Table, self).renderTable()
        return self.no_rows_message



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: sources.py 913 2009-01-02 12:47:38Z mac $

import stabledict
import zc.sourcefactory.basic
import zc.sourcefactory.contextual
import zope.component

import icemac.addressbook.export.interfaces

from icemac.addressbook.i18n import MessageFactory as _


class TitleMappingSource(zc.sourcefactory.basic.BasicSourceFactory):
    "Abstract base class for sources using a mapping between value and title."
    
    _mapping = None # to be set in child class

    def getValues(self):
        return self._mapping.keys()

    def getTitle(self, value):
        return self._mapping[value]


class SexSource(TitleMappingSource):
    _mapping = stabledict.StableDict(
        ((u'male', _(u'male')),
         (u'female', _(u'female'))))


class SalutationSource(TitleMappingSource):
    _mapping = stabledict.StableDict(
        ((u'male', _(u'Mr.')), 
         (u'female', _(u'Ms.'))))

salutation_source = SalutationSource()


class PhoneNumberKindSource(TitleMappingSource):
    _mapping = stabledict.StableDict((
            (u'cell phone', _(u'cell phone')),
            (u'private fax', _(u'private fax')),
            (u'private phone', _(u'private phone')),
            (u'work fax', _(u'work fax')),
            (u'work phone', _(u'work phone')),
            (u'other', _(u'other')),
            ))

phone_number_kind_source = PhoneNumberKindSource()


class WorkPrivateKindSource(TitleMappingSource):
    _mapping = stabledict.StableDict(
        ((u'private', _(u'private')),
         (u'work', _(u'work')),
         (u'other', _(u'other'))))


work_private_kind_source = WorkPrivateKindSource()


class KeywordSource(zc.sourcefactory.basic.BasicSourceFactory):

    def getValues(self):
        import icemac.addressbook.interfaces # avoid circular import
        keywords = zope.component.getUtility(
            icemac.addressbook.interfaces.IKeywords)
        return keywords.get_keywords()

    def getTitle(self, value):
        return value.title

keyword_source = KeywordSource()


class ContextByInterfaceSource(
    zc.sourcefactory.contextual.BasicContextualSourceFactory):
    "Source to select objects from context container by a given interface."

    def __init__(self, interface):
        super(ContextByInterfaceSource, self).__init__()
        self.interface = interface

    def getValues(self, context):
        for value in context.values():
            if self.interface.providedBy(value):
                yield value
    
    def getTitle(self, context, value):
        import icemac.addressbook.interfaces # avoid circular import
        return icemac.addressbook.interfaces.ITitle(value)


class ExporterSource(zc.sourcefactory.basic.BasicSourceFactory):

    def getValues(self):
        return zope.component.getAllUtilitiesRegisteredFor(
            icemac.addressbook.export.interfaces.IExporter)

    def getTitle(self, value):
        return u'%s (%s)' % (value.title, value.description)

exporter_source = ExporterSource()



# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: __init__.py 1075 2009-03-21 09:29:00Z mac $
"""Database initialisation and upgrading."""

import zope.app.generations.generations


manager = zope.app.generations.generations.SchemaManager(
    minimum_generation=4,
    generation=4,
    package_name='icemac.addressbook.generations')



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: evolve4.py 1075 2009-03-21 09:29:00Z mac $

__docformat__ = "reStructuredText"

import zope.app.zopeappgenerations

generation = 4


def evolve(context):
    """Update the root folder to be a ``zope.site.folder.Folder`` instead of
       ``zope.app.folder.folder.Folder``.
    """
    root = zope.app.zopeappgenerations.getRootFolder(context)
    context.connection.root()._p_changed = True
    root._p_changed = True



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: person.py 997 2009-02-03 16:58:46Z mac $

import gocept.reference
import zope.annotation.interfaces
import zope.container.btree
import zope.interface
import zope.lifecycleevent
import zope.schema.fieldproperty

import icemac.addressbook.interfaces
import icemac.addressbook.sources

class Person(zope.container.btree.BTreeContainer):
    "A person."

    zope.interface.implements(icemac.addressbook.interfaces.IPerson,
                              icemac.addressbook.interfaces.IPersonDefaults,
                              zope.annotation.interfaces.IAttributeAnnotatable)

    first_name = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPerson['first_name'])
    last_name = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPerson['last_name'])
    birth_date = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPerson['birth_date'])
    sex = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPerson['sex'])
    notes = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IPerson['notes'])

    keywords = gocept.reference.ReferenceCollection(ensure_integrity=True)
    default_postal_address = gocept.reference.Reference(ensure_integrity=True)
    default_email_address = gocept.reference.Reference(ensure_integrity=True)
    default_home_page_address = gocept.reference.Reference(
        ensure_integrity=True)
    default_phone_number = gocept.reference.Reference(ensure_integrity=True)


@zope.component.adapter(icemac.addressbook.interfaces.IPerson)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def title(person):
    """Whole name of the person."""
    if not(person.first_name or person.last_name):
        return u'<no name>'
    if not person.first_name:
        if not person.sex:
            return person.last_name
        else:
            return u'%s %s' % (
                icemac.addressbook.sources.salutation_source.factory.
                    getTitle(person.sex),
                person.last_name)
    return u'%s, %s' % (person.last_name, person.first_name)


@zope.component.adapter(
    icemac.addressbook.interfaces.IPerson,
    zope.lifecycleevent.IObjectCreatedEvent)
def person_created(person, event):
    """Create initial infrastructure or update existing infrastructure to
    current requirements (using generation)."""
    # set default values for references as z3c.form accesses the
    # attributes before a value is assigned and gets an AttributeError
    # otherwise
    if not hasattr(person, 'keywords'):
        person.keywords = set()
    for attrib in ['default_postal_address', 'default_email_address',
                   'default_home_page_address', 'default_phone_number']:
        if not hasattr(person, attrib):
            setattr(person, attrib, None)


class Keywords(object):

    zope.interface.implements(icemac.addressbook.interfaces.IKeywords)
    zope.component.adapts(icemac.addressbook.interfaces.IPerson)

    def __init__(self, context):
        self.context = context

    def get_keywords(self):
        return self.context.keywords

    def get_titles(self):
        return [x.title for x in self.get_keywords()]



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: menu.py 913 2009-01-02 12:47:38Z mac $

import zope.viewlet.manager
import z3c.menu.ready2go
import z3c.menu.ready2go.manager


SearchMenu = zope.viewlet.manager.ViewletManager(
    'left', z3c.menu.ready2go.IContextMenu, 
    bases=(z3c.menu.ready2go.manager.MenuManager,))



# -*- coding: utf-8 -*-
# Copyright (c) 2009 Michael Howitz
# See also LICENSE.txt
# $Id: testing.py 1084 2009-03-29 13:53:11Z mac $

import doctest
import icemac.addressbook.address
import icemac.addressbook.addressbook
import icemac.addressbook.keyword
import icemac.addressbook.person
import icemac.addressbook.principals.principals
import icemac.addressbook.principals.sources
import icemac.addressbook.utils
import os.path
import zope.annotation.attribute
import zope.app.testing.functional
import zope.testing.cleanup
import zope.testing.testrunner.layer
import zope.testbrowser.browser
import zope.testbrowser.interfaces


class AddressBookUnitTests(zope.testing.testrunner.layer.UnitTests):
    """Layer for gathering addressbook unit tests."""

    @classmethod
    def setUp(cls):
        # some classes use gocept.reference and store default values for
        # which annotations are needed.
        zope.component.provideAdapter(
            zope.annotation.attribute.AttributeAnnotations)

    @classmethod
    def tearDown(cls):
        zope.testing.cleanup.tearDown()


# XXX zope.app.testing.functional.defineLayer benutzen
ftesting_zcml = os.path.join(os.path.dirname(__file__), 'ftesting.zcml')
FunctionalLayer = zope.app.testing.functional.ZCMLLayer(
    ftesting_zcml, __name__, 'FunctionalLayer')


# def FunctionalDocTestSuite(module=None, **kw):
#     module = zope.testing.doctest._normalize_module(module)
#     suite = zope.app.testing.functional.FunctionalDocTestSuite(module, **kw)
#     suite.layer = FunctionalLayer
#     return suite


def FunctionalDocFileSuite(*paths, **kw):
    kw['optionflags'] = (kw.get('optionflags', 0) |
                         doctest.ELLIPSIS |
                         doctest.NORMALIZE_WHITESPACE)
    suite = zope.app.testing.functional.FunctionalDocFileSuite(*paths, **kw)
    suite.layer = FunctionalLayer
    return suite


class FunctionalTestCase(zope.app.testing.functional.FunctionalTestCase):
    layer = FunctionalLayer


def _get_control_names(interface, browser=None, form=None):
    """Get a sorted list of names of controls providing the given interface.

    Whether the browser was given as argument the form of the browser is used.
    """
    if browser is not None:
        form = browser.getForm()
    else:
        browser = form.browser
    names = []
    for ctrl in form.mech_form.controls:
        control = zope.testbrowser.browser.controlFactory(ctrl, form, browser)
        if interface.providedBy(control):
            names.append(control.name)
    return sorted(names)

def get_submit_control_names(browser=None, form=None):
    """Get a list of the names of the submit controls in the form.

    Whether the browser was given as argument the form of the browser is used.
    """
    return _get_control_names(
        zope.testbrowser.interfaces.ISubmitControl, browser, form)


def get_all_control_names(browser=None, form=None):
    """Get a list of all names of the controls in the form.

    Whether the browser was given as argument the form of the browser is used.
    """
    return _get_control_names(
        zope.testbrowser.interfaces.IControl, browser, form)


def create_addressbook(parent=None, name='ab', title=u'test address book'):
    ab = icemac.addressbook.utils.create_obj(
        icemac.addressbook.addressbook.AddressBook, title=title)
    if parent is None:
        parent = zope.app.testing.functional.getRootFolder()
    parent[name] = ab
    return ab


def create_keyword(addressbook, title, notes=u''):
    parent = addressbook.keywords
    name = icemac.addressbook.utils.create_and_add(
        parent, icemac.addressbook.keyword.Keyword, title=title, notes=notes)
    return parent[name]


@icemac.addressbook.utils.set_site
def create_person(parent, last_name, return_obj=True, **kw):
    kw['last_name'] = last_name
    name = icemac.addressbook.utils.create_and_add(
        parent, icemac.addressbook.person.Person, **kw)
    if return_obj:
        return parent[name]


@icemac.addressbook.utils.set_site
def create_postal_address(person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.PostalAddress, **kw)
    address = person[name]
    if set_as_default:
        person.default_postal_address = address
    if return_obj:
        return address


@icemac.addressbook.utils.set_site
def create_email_address(person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.EMailAddress, **kw)
    address = person[name]
    if set_as_default:
        person.default_email_address = address
    if return_obj:
        return address


@icemac.addressbook.utils.set_site
def create_home_page_address(
    person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.HomePageAddress, **kw)
    address = person[name]
    if set_as_default:
        person.default_home_page_address = address
    if return_obj:
        return address


@icemac.addressbook.utils.set_site
def create_phone_number(person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.PhoneNumber, **kw)
    number = person[name]
    if set_as_default:
        person.default_phone_number = number
    if return_obj:
        return number

@icemac.addressbook.utils.set_site
def create_user(ab, first_name, last_name, email, password, roles):
    person = create_person(
        ab, ab, last_name, first_name=first_name)
    create_email_address(ab, person, email=email)

    role_factory = icemac.addressbook.principals.sources.role_source.factory
    role_values = role_factory.getValues()
    selected_roles = []
    for role_title in roles:
        for candidate in role_values:
            if role_factory.getTitle(candidate) == role_title:
                selected_roles.append(candidate)
                break
    icemac.addressbook.utils.create_and_add(
        ab.principals, icemac.addressbook.principals.principals.Principal,
        person=person, password=password, roles=selected_roles)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: simple.py 1093 2009-03-31 19:17:47Z mac $

from cStringIO import StringIO
from icemac.addressbook.address import interface_title_prefix_class_mapping
from icemac.addressbook.i18n import MessageFactory as _
import datetime
import icemac.addressbook.export.interfaces
import icemac.addressbook.interfaces
import xlwt
import zope.interface


# fonts
head_font = xlwt.Font()
head_font.name = 'Courier'

group_font = xlwt.Font()
group_font.name = 'Courier'
group_font.bold = True


# styles
default_style = xlwt.XFStyle()

head_style = xlwt.XFStyle()
head_style.font = head_font

group_style = xlwt.XFStyle()
group_style.font = group_font

date_style = xlwt.XFStyle()
date_style.num_format_str = 'DD.MM.YY'


value_style_mapping = {
    datetime.date: date_style
    }


def convert_value(value):
    """Convert the value for xls export."""
    if value is None:
        return value
    if value.__class__ in (str, unicode, float, int, datetime.date):
        return value
    if hasattr(value, '__iter__'):
        return ', '.join(convert_value(v) for v in value)
    return icemac.addressbook.interfaces.ITitle(value)


class XLSExport(object):
    """Abstract base class for xls export."""

    zope.interface.implements(icemac.addressbook.export.interfaces.IExporter)

    file_extension = 'xls'
    mime_type = 'application/vnd.ms-excel'
    title = None # to be set in subclass
    description = None # to be set in subclass

    def export(self, *persons):
        self.workbook = xlwt.Workbook()
        self.sheet = self.workbook.add_sheet(_(u'Address book - Export'))
        self.persons = persons
        self.col = 0

        self._export()

        io = StringIO()
        self.workbook.save(io)
        return io.getvalue()

    def write_headlines(self, col, interface, headline):
        self.sheet.write(0, col, headline, group_style)
        for name, field in zope.schema.getFieldsInOrder(interface):
            self.sheet.write(1, col, field.title, head_style)
            col += 1
        return col

    def write_cell(self, row, col, value):
        style = value_style_mapping.get(value.__class__, default_style)
        self.sheet.write(row, col, convert_value(value), style)

    def write_col(self, interface, obj_getter, headline):
        max_col = self.write_headlines(self.col, interface, headline)
        row = 1
        for person in self.persons:
            row += 1
            obj = obj_getter(person)
            self.write_obj(row, self.col, interface, obj)
        self.col = max_col

    def write_obj(self, row, col, interface, obj):
        if obj is None:
            return col
        idx = 0
        for name, field in zope.schema.getFieldsInOrder(interface):
            self.write_cell(row, col + idx, getattr(obj, name))
            idx += 1
        return col + idx

    def write_person_data(self):
        self.write_col(
            icemac.addressbook.interfaces.IPerson, lambda x:x, _('person'))


class DefaultsExport(XLSExport):

    title = _(u'XLS main')
    description = _(
        u'Exports person data and main addresses resp. phone numbers.')

    def _export(self):
        self.write_person_data()
        for iface, title, prefix, dummy in interface_title_prefix_class_mapping:
            self.write_col(iface, lambda x:getattr(x, 'default_'+prefix), title)


class CompleteExport(XLSExport):

    title = _(u'XLS complete')
    description = _(
        u'Exports person data and all addresses resp. phone numbers.')

    def _export(self):
        self.write_person_data()
        for iface, title, prefix, dummy in interface_title_prefix_class_mapping:
            self.write_block(iface, prefix, title)

    def write_block(self, iface, prefix, title):
        num_blocks = 0
        blocks_with_header = 0
        start_col = max_col = self.col
        row = 1
        for person in self.persons:
            row += 1
            # write default objs first
            default_obj = getattr(person, 'default_' + prefix)
            if default_obj is None:
                continue
            if num_blocks == 0:
                max_col = self.write_headlines(
                    start_col, iface, _('main '+title))
                num_blocks = 1
                blocks_with_header = 1
            col = self.write_obj(row, start_col, iface, default_obj)
            if col == start_col:
                # nothing written because there was no obj
                continue
            # write other objs after default obj
            objs = [obj
                    for obj in icemac.addressbook.utils.iter_by_interface(
                        person, iface)
                    if obj != default_obj]
            if not objs:
                continue
            num_blocks = max(num_blocks, len(objs) + 1)
            while blocks_with_header < num_blocks:
                max_col = self.write_headlines(
                    max_col, iface, _('other '+title))
                blocks_with_header += 1
            for obj in objs:
                col = self.write_obj(row, col, iface, obj)
        self.col = max_col



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: tests.py 1080 2009-03-26 17:41:06Z mac $

import gocept.reference.verify
import unittest
import zope.interface.verify

import icemac.addressbook.testing
import icemac.addressbook.principals.interfaces
import icemac.addressbook.principals.principals


class TestInterfaces(unittest.TestCase):

    def test_principal(self):
        principal = icemac.addressbook.principals.principals.Principal()
        # need to call created event handler here, because person
        # attribute is a descriptor wrapping the one verifyObject
        # expects.
        icemac.addressbook.principals.principals.created(principal, None)
        gocept.reference.verify.verifyObject(
            icemac.addressbook.principals.interfaces.IPrincipal, principal)
        zope.interface.verify.verifyObject(
            icemac.addressbook.principals.interfaces.IPasswordFields, principal)
        zope.interface.verify.verifyObject(
            icemac.addressbook.principals.interfaces.IRoles, principal)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestInterfaces))
    suite.layer = icemac.addressbook.testing.AddressBookUnitTests
    return suite



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: tests.py 935 2009-01-08 10:19:09Z mac $

import unittest
import zope.interface.verify

import icemac.addressbook.export.interfaces
import icemac.addressbook.export.xls.simple
import icemac.addressbook.testing


class TestInterfaces(unittest.TestCase):

    def test_xls(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.export.interfaces.IExporter,
            icemac.addressbook.export.xls.simple.XLSExport())
        zope.interface.verify.verifyObject(
            icemac.addressbook.export.interfaces.IExporter,
            icemac.addressbook.export.xls.simple.DefaultsExport())
        zope.interface.verify.verifyObject(
            icemac.addressbook.export.interfaces.IExporter,
            icemac.addressbook.export.xls.simple.CompleteExport())


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestInterfaces))
    suite.layer = icemac.addressbook.testing.AddressBookUnitTests
    return suite



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: i18n.py 913 2009-01-02 12:47:38Z mac $

"""I18n message factory for icemac.addressbook. """

import zope.i18nmessageid

MessageFactory = zope.i18nmessageid.MessageFactory('icemac.addressbook')



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: base.py 1081 2009-03-28 20:03:50Z mac $

from icemac.addressbook.i18n import MessageFactory as _
import icemac.addressbook.browser.interfaces
import icemac.addressbook.interfaces
import icemac.addressbook.utils
import transaction
import z3c.form.button
import z3c.form.field
import z3c.form.group
import z3c.formui.form
import zope.app.publication.browser
import zope.component
import zope.interface
import zope.security
import zope.security.interfaces
import zope.traversing.api
import zope.traversing.browser.interfaces


class display_title(object):

    zope.component.adapts(
        zope.interface.Interface,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)
    zope.interface.implements(icemac.addressbook.interfaces.ITitle)

    def __init__(self, context, request):
        self.context = context

    def __call__(self):
        return icemac.addressbook.interfaces.ITitle(self.context)


def create(form, class_, data):
    obj = icemac.addressbook.utils.create_obj(class_)
    z3c.formui.form.applyChanges(form, obj, data)
    return obj


def all_(*constraints):
    """All given button constraints must evaluate to true."""
    return lambda form: all(constraint(form) for constraint in constraints)


class BaseAddForm(z3c.formui.form.AddForm):
    """Simple base add form."""

    interface = None # interface for form
    class_ = None # create object from this class
    next_url = None # target after creation, one of ('object', 'parent')

    @property
    def fields(self):
        return z3c.form.field.Fields(self.interface)

    def create(self, data):
        return create(self, self.class_, data)

    def add(self, obj):
        try:
            self._name = icemac.addressbook.utils.add(self.context, obj)
        except zope.interface.Invalid, e:
            transaction.doom()
            raise z3c.form.interfaces.ActionExecutionError(e)

    @z3c.form.button.buttonAndHandler(_('Add'), name='add')
    def handleAdd(self, action):
        # because we define a new action we have to duplicate the
        # existing action because otherwise we'll lose it.
        super(BaseAddForm, self).handleAdd(self, action)

    @z3c.form.button.buttonAndHandler(_('Cancel'), name='cancel')
    def handleCancel(self, action):
        self.next_url = 'parent'
        self._finishedAdd = True

    def nextURL(self):
        if self.next_url == 'object':
            context = self.context[self._name]
        elif self.next_url == 'parent':
            context = self.context
        else:
            raise ValueError("Don't know how to handle next_url %r.")
        return zope.component.getMultiAdapter(
            (context, self.request),
            zope.traversing.browser.interfaces.IAbsoluteURL)()


class BaseEditForm(z3c.formui.form.EditForm):
    """Base Edit form."""

    next_url = None # target ojbect after edit, one of ('object', 'parent')
    next_view = None # target view after edit (None for default view)
    interface = None # interface for form

    @property
    def fields(self):
        return z3c.form.field.Fields(self.interface)

    def render(self):
        if self.status in (self.successMessage, self.noChangesMessage):
            self.redirect_to_next_url()
        else:
            return super(BaseEditForm, self).render()

    def redirect_to_next_url(self):
        if self.next_url == 'object':
            target = self.context
        elif self.next_url == 'parent':
            target = zope.traversing.api.getParent(self.context)
        else:
            raise ValueError('next_url %r unknown' % self.next_url)
        target_url = zope.component.getMultiAdapter(
            (target, self.request),
            zope.traversing.browser.interfaces.IAbsoluteURL)()
        if self.next_view:
            target_url += '/%s' % self.next_view
        self.request.response.redirect(target_url)

    def applyChanges(self, data):
        try:
            super(BaseEditForm, self).applyChanges(data)
        except zope.interface.Invalid, e:
            transaction.doom()
            raise z3c.form.interfaces.ActionExecutionError(e)


class BaseEditFormWithCancel(BaseEditForm):
    "BaseEditForm with cancel button."

    @z3c.form.button.buttonAndHandler(_('Apply'), name='apply')
    def handleApply(self, action):
        # because we define a new action we have to duplicate the
        # existing action because otherwise we'll lose it.
        super(BaseEditFormWithCancel, self).handleApply(self, action)

    @z3c.form.button.buttonAndHandler(_('Cancel'), name='cancel')
    def handleCancel(self, action):
        self.status = self.noChangesMessage


class BaseDeleteForm(BaseEditForm):
    "Display a deletion confirmation dialog."

    label = _(u'Do you really want to delete this entry?')
    interface = None
    field_names = () # tuple of field names for display or empty for all

    mode = z3c.form.interfaces.DISPLAY_MODE
    next_url = 'object'

    @property
    def fields(self):
        fields = z3c.form.field.Fields(self.interface)
        if self.field_names:
            fields = fields.select(*self.field_names)
        return fields

    @z3c.form.button.buttonAndHandler(_(u'No, cancel'), name='cancel')
    def handleCancel(self, action):
        self.status = self.noChangesMessage

    @z3c.form.button.buttonAndHandler(_(u'Yes, delete it'), name='delete')
    def handleDelete(self, action):
        self.next_url = 'parent'
        self.redirect_to_next_url()
        self._do_delete()
        return ''

    def _do_delete(self):
        name = zope.traversing.api.getName(self.context)
        parent = zope.traversing.api.getParent(self.context)
        del parent[name]


class PrefixGroup(z3c.form.group.Group):
    """Group which sets a prefix."""

    prefix = None # to be set in subclass
    interface = None # to be set in subclass

    @property
    def fields(self):
        return z3c.form.field.Fields(self.interface, prefix=self.prefix)


def can_access(uri_part):
    """Create a button condition function to test whether the user can
    access the URL context/@@absolute_url/<uri_part>."""
    def can_access_form(form):
        traverser = zope.app.publication.browser.PublicationTraverser()
        try:
            view = traverser.traverseRelativeURL(
                form.request, form.context, uri_part)
        except (zope.security.interfaces.Unauthorized,
                zope.security.interfaces.Forbidden,
                LookupError):
            return False
        else:
            # we're assuming that view pages are callable
            # this is a pretty sound assumption
            if not zope.security.canAccess(view, '__call__'):
                return False
        return True
    return can_access_form




# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: tests.py 935 2009-01-08 10:19:09Z mac $

import gocept.reference.verify
import unittest
import zope.annotation.attribute
import zope.component
import zope.interface.verify
import zope.testing.cleanup
import zope.testing.testrunner.layer

import icemac.addressbook.address
import icemac.addressbook.addressbook
import icemac.addressbook.interfaces
import icemac.addressbook.keyword
import icemac.addressbook.person
import icemac.addressbook.testing


class TestInterfaces(unittest.TestCase):

    def test_person(self):
        person = icemac.addressbook.person.Person()
        gocept.reference.verify.verifyObject(
            icemac.addressbook.interfaces.IPerson, person)

        gocept.reference.verify.verifyObject(
            icemac.addressbook.interfaces.IPersonDefaults, person)

    def test_address_book(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.interfaces.IAddressBook,
            icemac.addressbook.addressbook.AddressBook())

    def test_postal_address(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.interfaces.IPostalAddress,
            icemac.addressbook.address.PostalAddress())

    def test_email_address(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.interfaces.IEMailAddress,
            icemac.addressbook.address.EMailAddress())

    def test_home_page_address(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.interfaces.IHomePageAddress,
            icemac.addressbook.address.HomePageAddress())

    def test_phone_number(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.interfaces.IPhoneNumber,
            icemac.addressbook.address.PhoneNumber())

    def test_keywords(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.interfaces.IKeywords,
            icemac.addressbook.keyword.KeywordContainer())
        zope.interface.verify.verifyObject(
            icemac.addressbook.interfaces.IKeywords,
            icemac.addressbook.person.Keywords(None))

    def test_keyword(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.interfaces.IKeyword,
            icemac.addressbook.keyword.Keyword())


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestInterfaces))
    suite.layer = icemac.addressbook.testing.AddressBookUnitTests
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: simple.py 1047 2009-03-14 09:35:01Z mac $

import z3c.form.button
import z3c.ptcompat
import z3c.table.column
import z3c.table.table
import zc.sourcefactory.interfaces
import zope.interface
import zope.schema
import zope.session.interfaces

import icemac.addressbook.browser.base
import icemac.addressbook.interfaces

from icemac.addressbook.i18n import MessageFactory as _


class IExporterChoice(zope.interface.Interface):

    exporter = zope.schema.Choice(
        title=_(u'Export using'),
        source=icemac.addressbook.sources.exporter_source)


class ExportForm(icemac.addressbook.browser.base.BaseEditForm):

    ignoreContext = True
    interface = IExporterChoice
    template = z3c.ptcompat.ViewPageTemplateFile('export.pt')

    def renderTable(self):
        table = PersonTable(self.__parent__, self.request)
        table.update()
        return table.render()

    @z3c.form.button.buttonAndHandler(_('Export'), name='export')
    def handleExport(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        session = zope.session.interfaces.ISession(self.request)[
            icemac.addressbook.interfaces.PACKAGE_ID]
        session['person_ids'] = self.request.form.get('persons', ())
        session['exporter_token'] = zc.sourcefactory.interfaces.IToken(
            data['exporter'])
        self.request.response.redirect('@@export.html')


class PersonTable(z3c.table.table.Table):
    "Person table viewlet."

    def update(self):
        self.result = self.context.result
        if self.result is not None:
            # only render table when a search happend
            super(PersonTable, self).update()

    @property
    def values(self):
        return self.result


class CheckBoxColumn(z3c.table.column.CheckBoxColumn):

    header = u''
    weight = 1

    def getItemKey(self, item):
        return 'persons:list'

    def isSelected(self, item):
        if self.request.get(self.getItemKey(item), None) is None:
            # not in request, return default: selected
            return True
        return super(CheckBoxColumn, self).isSelected(item)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: edit.py 913 2009-01-02 12:47:38Z mac $

import icemac.addressbook.interfaces
import icemac.addressbook.browser.base
from icemac.addressbook.i18n import MessageFactory as _


class EditForm(icemac.addressbook.browser.base.BaseEditFormWithCancel):

    label = _(u'Edit address book data')
    interface = icemac.addressbook.interfaces.IAddressBook
    next_url = 'object'
    next_view = '@@edit.html'



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: adapter.py 997 2009-02-03 16:58:46Z mac $

import zope.interface
import zope.schema.interfaces
import zope.location.interfaces
import zope.site.hooks

import icemac.addressbook.interfaces
import icemac.addressbook.principals.interfaces


@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def gocept_country_title(obj):
    """Title for objects from gocept.country."""
    return obj.name


@zope.component.adapter(zope.schema.interfaces.IChoice,
                        zope.interface.Interface)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def title_for_choice_value(field, value):
    "Get the title for a value of a field which is a Choice."
    factory = field.source.factory
    return factory.getTitle(value)


@zope.component.adapter(zope.interface.Interface)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def default_title(obj):
    "Default title adapter which returns str represantation of obj."
    if isinstance(obj, basestring):
        return obj
    return str(obj)


@zope.component.adapter(zope.interface.Interface)
@zope.interface.implementer(icemac.addressbook.principals.interfaces.IRoot)
def principal_root(principal):
    return zope.site.hooks.getSite()



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: interfaces.py 1069 2009-03-20 17:23:09Z mac $

import z3c.form.interfaces
import z3c.formui.interfaces
import z3c.layer.pagelet
import z3c.menu.ready2go


class IMainMenu(z3c.menu.ready2go.ISiteMenu):
    """Main menu."""


class IAddMenu(z3c.menu.ready2go.IAddMenu):
    """Add menu."""



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: root.py 1066 2009-03-20 17:13:24Z mac $

import z3c.pagelet.browser
import icemac.addressbook.interfaces

from icemac.addressbook.i18n import MessageFactory as _


class FrontPage(z3c.pagelet.browser.BrowserPagelet):
    """Pagelet for the front page."""

    def getAddressBooks(self):
        return [value
                for value in self.context.values()
                if icemac.addressbook.interfaces.IAddressBook.providedBy(value)
                ]

    def countEntries(self, address_book):
        count = len(address_book)
        if count == 1: # XXX use i18n instead!
            entries = _(u'entry')
        else:
            entries = _(u'entries')
        return "%s %s" % (count, entries)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: interfaces.py 938 2009-01-08 10:20:58Z mac $

import gocept.country
import gocept.country.db
import re
import zope.interface
import zope.schema

import icemac.addressbook.sources
from icemac.addressbook.i18n import MessageFactory as _


class StateWithoutCountryError(zope.schema.ValidationError):
    __doc__ = _("""You can't select a state without selecting a country.""")


class StateCountryMismatchError(zope.schema.ValidationError):
    __doc__ = _(
        "You selected a state which does not belong to the selected country.")


PACKAGE_ID = 'icemac.addressbook'


class ITitle(zope.interface.Interface):
    """Title of an entity."""

    def __str__():
        """Return the title of the entity."""


class IAddressBook(zope.interface.Interface):
    """An address book."""

    keywords = zope.interface.Attribute(u'keywords collection (IKeywords).')
    principals = zope.interface.Attribute(
        u'zope.app.authentication.interfaces.IInternalPrincipalContainer')

    title = zope.schema.TextLine(title=_(u'title'))
    notes = zope.schema.Text(title=_(u'notes'), required=False)


class IPerson(zope.interface.Interface):
    """A person."""

    first_name = zope.schema.TextLine(title=_(u'first name'), required=False)
    last_name = zope.schema.TextLine(title=_(u'last name'))
    birth_date = zope.schema.Date(title=_(u'birth date'), required=False)
    sex = zope.schema.Choice(
        title=_(u'sex'), source=icemac.addressbook.sources.SexSource(),
        required=False)
    keywords = zope.schema.Set(
        title=_('keywords'), required=False,
        value_type=zope.schema.Choice(
            title=_('keywords'),
            source=icemac.addressbook.sources.keyword_source))
    notes = zope.schema.Text(title=_(u'notes'), required=False)


class IPostalAddress(zope.interface.Interface):
    """A postal address."""

    kind = zope.schema.Choice(
        title=_(u'kind'), required=False,
        source=icemac.addressbook.sources.work_private_kind_source)
    address_prefix = zope.schema.TextLine(
        title=_(u'address prefix'), required=False,
        description=_(u'e. g. company name or c/o'))
    street = zope.schema.TextLine(title=_(u'street'), required=False)
    city = zope.schema.TextLine(title=_(u'city'), required=False)
    zip = zope.schema.TextLine(title=_(u'zip'), required=False)
    country = zope.schema.Choice(
        title=_(u'country'), source=gocept.country.countries,
        required=False, default=gocept.country.db.Country('DE'))
    state = zope.schema.Choice(
        title=_(u'state'), required=False,
        source=gocept.country.SubdivisionSource(country_code=['DE']))
    notes = zope.schema.Text(title=_(u'notes'), required=False)

    @zope.interface.invariant
    def state_country(obj):
        if not obj.state:
            return None
        if obj.state and not obj.country:
            raise StateWithoutCountryError(obj)
        if obj.country.alpha2 != obj.state.country_code:
            raise StateCountryMismatchError(obj)


class IEMailAddress(zope.interface.Interface):
    """An e-mail address."""

    kind = zope.schema.Choice(
        title=_(u'kind'), required=False,
        source=icemac.addressbook.sources.work_private_kind_source)
    email = zope.schema.TextLine(
        title=_(u'e-mail address'), required=False,
        constraint=re.compile(
            "^[=+A-Za-z0-9_.-]+@([A-Za-z0-9_\-]+\.)+[A-Za-z]{2,6}$").match)
    notes = zope.schema.Text(title=_(u'notes'), required=False)


class IHomePageAddress(zope.interface.Interface):
    """A home page address."""

    kind = zope.schema.Choice(
        title=_(u'kind'), required=False,
        source=icemac.addressbook.sources.work_private_kind_source)
    url = zope.schema.URI(title=_(u'URL'), required=False)
    notes = zope.schema.Text(title=_(u'notes'), required=False)


class IPhoneNumber(zope.interface.Interface):
    """A phone number."""

    kind = zope.schema.Choice(
        title=_(u'kind'), required=False,
        source=icemac.addressbook.sources.phone_number_kind_source)
    number = zope.schema.TextLine(title=_(u'number'), required=False)
    notes = zope.schema.Text(title=_(u'notes'), required=False)


class IPersonDefaults(zope.interface.Interface):
    """Default addresses, phone numbers etc."""

    default_postal_address = zope.schema.Choice(
        title=_(u'main postal address'),
        source=icemac.addressbook.sources.ContextByInterfaceSource(
            IPostalAddress))
    default_phone_number = zope.schema.Choice(
        title=_(u'main phone number'),
        source=icemac.addressbook.sources.ContextByInterfaceSource(
            IPhoneNumber))
    default_email_address = zope.schema.Choice(
        title=_(u'main e-mail address'),
        source=icemac.addressbook.sources.ContextByInterfaceSource(
            IEMailAddress))
    default_home_page_address = zope.schema.Choice(
        title=_(u'main home page address'),
        source=icemac.addressbook.sources.ContextByInterfaceSource(
            IHomePageAddress))


class IKeywords(zope.interface.Interface):
    """Collection of keywords."""

    def get_keywords():
        """Get the keywords in the collection."""

    def get_titles():
        """Get the titles of the keywords in the collection."""


class IKeyword(zope.interface.Interface):
    """A keyword."""

    title = zope.schema.TextLine(title=_(u'keyword'))
    notes = zope.schema.Text(title=_(u'notes'), required=False)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: ftests.py 1104 2009-03-31 19:53:53Z mac $

import icemac.addressbook.testing


def test_suite():
    return icemac.addressbook.testing.FunctionalDocFileSuite(
        'adapter.txt',
        'address.txt',
        'catalog.txt',
        'export/export.txt',
        'person.txt',
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt

import os.path
import setuptools

def read(*path_elements):
    return "\n\n" + file(os.path.join(*path_elements)).read()

version = '0.3.2'

setuptools.setup(
    name='icemac.addressbook',
    version=version,
    description="Multi user address book application",
    long_description=(
        read('README.txt') +
        read('INSTALL.txt') +
        read('TODO.txt') +
        read('CHANGES.txt')
        ),
    keywords='python address addressbook zope3 zope application web phone '
             'number e-mail email home page homepage',
    author='Michael Howitz',
    author_email='icemac@gmx.net',
    url='http://pypi.python.org/pypi/icemac.addressbook',
    license='ZPL 2.1',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Paste',
        'Framework :: Zope3',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Other Audience',
        'Intended Audience :: Religion',
        'Intended Audience :: Telecommunications Industry',
        'License :: OSI Approved',
        'License :: OSI Approved :: Zope Public License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.5',
        'Topic :: Communications',
        'Topic :: Communications :: Email',
        'Topic :: Communications :: Email :: Address Book',
        'Topic :: Communications :: Telephony',
        'Topic :: Internet',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Topic :: Office/Business',
        'Topic :: Office/Business :: Groupware',
        'Topic :: Religion',
        ],
    packages=setuptools.find_packages('src'),
    package_dir = {'': 'src'},
    namespace_packages = ['icemac'],
    include_package_data=True,
    exclude_package_data = {'': ['DE_TODO.txt']},
    zip_safe=False,
    install_requires=[
        'setuptools',
        'zope.interface',
        'ZODB3',
        'ZConfig',
        'zdaemon',
        'zope.publisher',
        'zope.traversing',
        'zope.app.wsgi',
        'zope.app.appsetup',
        'zope.app.authentication',
        'zope.session',
        'zope.app.zopeappgenerations',
        'zope.i18n',
        'zope.app.principalannotation',
        # Packages required by application
        'zc.sourcefactory',
        'zope.securitypolicy',
        'zope.app.generations',
        'z3c.form',
        'z3c.formui',
        'z3c.layer.pagelet',
        'z3c.pagelet',
        'z3c.ptcompat',
        'gocept.reference',
        'StableDict',
        'gocept.country',
        'gocept.pagelet',
        'z3c.menu.ready2go',
        'zc.catalog',
        'xlwt',
        'z3c.table',
        'classproperty',
        'icemac.truncatetext',
        'z3c.pt',
        'zope.app.folder', # XXX required by older installations,
                           # deprecated and converted in 0.3
                           # (evolve4.py), remove in version 0.6 (Add
                           # notice to CHANGES.txt then, to show what
                           # to do to upgrade when evolve to
                           # generation 4 fails.)
        ],
    extras_require = dict(
        test=['zope.testing',
              'zope.app.testing',
              'zope.testbrowser',
              'xlrd',
              ]),
    entry_points = """
      [console_scripts]
      debug = icemac.addressbook.startup:interactive_debug_prompt
      addressbook = icemac.addressbook.startup:zdaemon_controller
      debug_ajax = icemac.addressbook.startup:zdaemon_controller_debug_ajax
      debug_pdb = icemac.addressbook.startup:zdaemon_controller_debug_pdb
      [paste.app_factory]
      main = icemac.addressbook.startup:application_factory
      """
    )



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: interfaces.py 913 2009-01-02 12:47:38Z mac $

import z3c.menu.ready2go
import zope.interface
import zope.viewlet.interfaces


class ISearchMenu(z3c.menu.ready2go.ISiteMenu):
    """Search menu."""


class ISearchForm(zope.viewlet.interfaces.IViewletManager):
    """Search form manager."""


class ISearchResult(zope.viewlet.interfaces.IViewletManager):
    """Search form manager."""


class ISearch(zope.interface.Interface):
    """A search."""
    
    def search(**kw):
        """Search for given keyword arguments and return iterable of results."""



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: base.py 913 2009-01-02 12:47:38Z mac $

import z3c.form.field
import z3c.formui.form
import z3c.form.button

import icemac.addressbook.browser.search.interfaces

from icemac.addressbook.i18n import MessageFactory as _


class BaseView(object):

    search_params = None

    @property
    def result(self):
        if not self.search_params:
            return
        return self.search()

    def search(self):
        search = icemac.addressbook.browser.search.interfaces.ISearch(self)
        return search.search(**self.search_params)


class BaseSearchForm(z3c.formui.form.Form):
    
    interface = None # to be set in child class

    ignoreContext = True
    formErrorsMessage = _('There were some errors.')

    @property
    def fields(self):
        return z3c.form.field.Fields(self.interface)

    @z3c.form.button.buttonAndHandler(_('Search'), name='search')
    def search(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        self.__parent__.search_params = data



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: simple_single_keyword.py 1001 2009-02-07 08:10:56Z mac $

import zope.catalog.interfaces
import zope.component
import zope.interface
import zope.schema

import icemac.addressbook.browser.search.base
import icemac.addressbook.sources
import icemac.addressbook.browser.search.interfaces

from icemac.addressbook.i18n import MessageFactory as _


class SearchView(icemac.addressbook.browser.search.base.BaseView):
    pass


class Search(object):
    """Search for a single keyword."""
    zope.component.adapts(SearchView)
    zope.interface.implements(
        icemac.addressbook.browser.search.interfaces.ISearch)

    def __init__(self, *args):
        pass

    def search(self, keyword):
        keyword = keyword.title
        catalog = zope.component.getUtility(zope.catalog.interfaces.ICatalog)
        result_set = catalog.searchResults(keywords={'any_of': (keyword, )})
        return result_set


class ISingleSimpleKeyword(zope.interface.Interface):

    keyword = zope.schema.Choice(
        title=_('keyword'),
        source=icemac.addressbook.sources.keyword_source)


class SearchForm(icemac.addressbook.browser.search.base.BaseSearchForm):

    interface = ISingleSimpleKeyword



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2009 Michael Howitz
# See also LICENSE.txt
# $Id: interfaces.py 1069 2009-03-20 17:23:09Z mac $

import z3c.form.interfaces
import z3c.formui.interfaces
import z3c.layer.pagelet
import z3c.menu.ready2go


class IAddressBookLayer(
    z3c.form.interfaces.IFormLayer, z3c.layer.pagelet.IPageletBrowserLayer):
    """Address book browser layer with form support."""


class IAddressBookBrowserSkin(
    z3c.formui.interfaces.IDivFormLayer, IAddressBookLayer):
    """The address book browser skin using the div-based layout."""
